#ifndef __DOMAIN__
#define __DOMAIN__

using XMLIOSERVER::XML::XMLNode;
using XMLIOSERVER::XML::THashAttributes;

namespace XMLIOSERVER
{
   class CDomain : public ObjectTemplate<CDomain>, public DomainAttribut
   {
      public:

         CDomain(void) : ObjectTemplate<CDomain>(), DomainAttribut()
         { /* Ne rien faire de plus */ }
         CDomain(const string& _id) : ObjectTemplate<CDomain>(_id), DomainAttribut()
         { /* Ne rien faire de plus */ }

         static string GetName(void) {return ("domain"); }

         inline void check(void) ;

         ~CDomain(void) { /* Ne rien faire de plus */ }

   }; // class Domain

   void CDomain::check(void)
   {
      static bool isChecked = false ;

      if (isChecked) return; // Si la vérification a été faite, on arrête tout.

      // domaine global
      if (!ni_glo.hasValue() || ni_glo<=0 ) throw XMLIOSERVER::XMLIOUndefinedValueException("domaine global mal defini") ;
      if (!nj_glo.hasValue() || nj_glo<=0 ) throw XMLIOSERVER::XMLIOUndefinedValueException("domaine global mal defini") ;

      //domaine local en i
      if ( ni.hasValue() && ibegin.hasValue())
      {
         if (iend.hasValue() && iend!=ibegin+ni-1) throw XMLIOSERVER::XMLIOUndefinedValueException("domain local mal défini 1") ;
         iend = ibegin + ni - 1 ;
      }
      else if (ni.hasValue() && iend.hasValue())
      {
         if (ibegin != ni-iend+1) throw XMLIOSERVER::XMLIOUndefinedValueException("domain local mal défini 2") ;
         ibegin = ni - iend + 1 ;
      }
      else if (ibegin.hasValue() && iend.hasValue())
      {
         if (ni!=iend-ibegin-1) throw XMLIOSERVER::XMLIOUndefinedValueException("domain local mal défini 3") ;
         ni = iend - ibegin - 1 ;
      }
      else throw XMLIOSERVER::XMLIOUndefinedValueException("domain local non défini 4") ;

      if (ni<0 || ibegin>iend || ibegin<1 || iend>ni_glo) throw XMLIOSERVER::XMLIOUndefinedValueException("domain local mal défini 5") ;

      //domaine local en j
      if ( nj.hasValue() && jbegin.hasValue())
      {
         if (jend.hasValue() && jend!=jbegin+nj-1) throw XMLIOSERVER::XMLIOUndefinedValueException("domain local mal défini 6") ;
         jend = jbegin + nj - 1 ;
      }
      else if (nj.hasValue() && jend.hasValue())
      {
         if (jbegin!=nj-jend+1) throw XMLIOSERVER::XMLIOUndefinedValueException("domain local mal défini 7") ;
         jbegin = nj - jend + 1 ;
      }
      else if (jbegin.hasValue() && jend.hasValue())
      {
         if (nj!=jend-jbegin-1) throw XMLIOSERVER::XMLIOUndefinedValueException("domain local mal défini 8") ;
         nj = jend - jbegin - 1 ;
      }
      else throw XMLIOSERVER::XMLIOUndefinedValueException("domain local non défini 9") ;

      if (nj<0 || jbegin>jend || jbegin<1 || jend>nj_glo) throw XMLIOSERVER::XMLIOUndefinedValueException("domain local mal défini 10") ;

      //cout<<"mask ni "<<mask->extent(0)<<" nj "<<mask->extent(1)<<"  "<<mask<<endl ;
      if (mask.hasValue())
         if (mask->extent(0) != ni || mask->extent(1) != nj)
            throw XMLIOSERVER::XMLIOUndefinedValueException("le champ mask n'a pas la même taille que le domaine local") ;

      if (data_dim.hasValue())
      {
         if (!(data_dim==1 || data_dim==2)) throw XMLIOSERVER::XMLIOUndefinedValueException("dimension non comptatible (doit être 1 ou 2)") ;
      }
      else throw XMLIOSERVER::XMLIOUndefinedValueException("Dimension des donnée non définie") ;

      if (data_ni.hasValue())
      {
         if (data_ni<=0) throw XMLIOSERVER::XMLIOUndefinedValueException("Dimension des données négative") ;
      }
      else
      {
         if (data_dim==1) data_ni=ni*nj ;
         else data_ni=(int) ni ;
      }

      if (!data_ibegin.hasValue()) data_ibegin=0 ;

      if (data_nj.hasValue())
      {
         if (data_nj<=0) throw XMLIOSERVER::XMLIOUndefinedValueException("Dimension des données négative") ;
      }
      else data_nj=(int) nj ;

      if (!data_jbegin.hasValue()) data_jbegin=0 ;

      if (!mask.hasValue())
      { // Si aucun masque n'est défini, on en crée un nouveau qui valide l'intégralité du domaine.
         mask->resize(shape(ni,nj)) ;
         *mask=true ;
      }

      // Gestion de la compression

      if (data_i_index.hasValue())
      {
         if (data_n_index.hasValue())
         {
            if (data_n_index!=data_i_index->extent(0))
               throw XMLIOSERVER::XMLIOUndefinedValueException("Dimension data_i_index incompatible avec data_n_index") ;
         }
         else data_n_index=data_i_index->extent(0) ;

         if (data_dim==2)
         {
            if (data_j_index.hasValue())
            {
               if (data_j_index->extent(0)!=data_i_index->extent(0))
                  throw XMLIOSERVER::XMLIOUndefinedValueException("Dimension data_j_index incompatible avec data_i_index") ;
            }
            else throw XMLIOSERVER::XMLIOUndefinedValueException("data_j_index non défini") ;
         }
      }
      else
      {
         if (data_n_index.hasValue() || (data_dim==2 && data_j_index.hasValue()))
            throw XMLIOSERVER::XMLIOUndefinedValueException("data_i_index non défini") ;
      }
      //cout<<"data_n_index.hasValue() "<<data_n_index.hasValue()<<endl ;
      if (!data_n_index.hasValue())
      {
         if (data_dim==1)
         {
            int i ;
            data_n_index=*data_ni ;
            //cout <<"data_n_index "<<data_n_index<<"  "<<data_ni<<" "<<data_nj<<endl ;

            data_i_index->resize(data_n_index) ;

            for (i=0;i<data_ni;i++) (*data_i_index)(i)=i ;
            //cout <<"data_i_index "<<*data_i_index<<endl ;
         }
         else   // data_dim=2
         {
            int i ;
            int j ;
            int count ;

            data_n_index = data_ni * data_nj ;
            data_i_index -> resize(data_n_index) ;
            data_j_index -> resize(data_n_index) ;
            for(count=0,i=0; i<data_ni;i++)
               for(j=0; j<data_nj;j++)
               {
                  (*data_i_index)(count)=i ;
                 // cout<<count<<"  "<<(*data_i_index)(count)<<endl ;
                  (*data_j_index)(count)=j ;
                  //cout<<count<<"  "<<(*data_j_index)(count)<<endl ;
                  count++ ;
               }
               //cout<<data_ni<<" "<<data_nj<<" "<<data_n_index<<" "<<*data_i_index<<" "<<*data_j_index<<endl ;
         }
      }

      isChecked = true ;
   }

} // namespace XMLIOSERVER

DECLARE_GROUP(Domain)

#endif // __FIELD__
